// ==============================================================
//                 ORBITER MODULE: doge1
//                  
//
// doge1.cpp
// Control module for doge1 vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include <math.h>
#include "OrbiterSoundSDK40.h"



// ==============================================================
// Some vessel parameters
// ==============================================================

const double MAIN_ISP = 20000;
const double MAIN_TH = 0.001;
const double FUEL_MASS = 0.3;
const double EMP_MASS = 13.7;

#define YELLOW RGB(255, 255, 80)
#define GREEN RGB(0, 225, 0)

double stiffness = 1e3;
double damping = 2e2;
static TOUCHDOWNVTX tdvtx_geardown[3] = {
	{_V( 0,      -0.075,  0.2), stiffness, damping, 1.0, 0.2},
	{_V(-0.1732, -0.075, -0.1), stiffness, damping, 1.0, 0.2},
	{_V( 0.1732, -0.075, -0.1), stiffness, damping, 1.0, 0.2}
};


class doge1: public VESSEL3 {
public:
	
	doge1 (OBJHANDLE hVessel, int flightmodel);
	~doge1 ();

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
//	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
//	bool clbkLoadGenericCockpit ();
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	

private:
	void DoKillRot(void);

	int MyID;
	THRUSTER_HANDLE th_main[1];
	PROPELLANT_HANDLE hProp;

	ATTACHMENTHANDLE hAttach;

	int kilrot_sw;


};

doge1::doge1 (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{

}

doge1::~doge1 ()
{

}





// ==============================================================
// doge1 class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void doge1::clbkSetClassCaps (FILEHANDLE cfg)
{

	kilrot_sw = 0;

	// physical specs
	SetCW (1, 1, 1, 1);
	SetRotDrag (_V(1,1,1));
	SetPMI (_V(0.1,0.1,0.1));
	SetCrossSections (_V(0.02,0.02,0.01));
	SetSize (0.2);
	SetEmptyMass (EMP_MASS);
	SetMeshVisibilityMode ((AddMesh ("nova_c/doge1")), MESHVIS_EXTERNAL);
	
	SetCameraOffset (_V(0,0,0));
	SetTouchdownPoints ( tdvtx_geardown, 3);
	

	//////////////////// attach //////////////////////////

	hAttach = CreateAttachment (true, _V( 0, 0, 0.0775), _V( 0,0,1), _V(0,1,0), "F9");

	hProp = CreatePropellantResource (FUEL_MASS, FUEL_MASS);

	
	// ***************** thruster definitions *******************

	th_main[0] = CreateThruster (_V( 0, 0, -0.19), _V( 0, 0, 1), MAIN_TH, hProp, MAIN_ISP);

	CreateThrusterGroup (&th_main[0], 1, THGROUP_MAIN);

	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("Exhaust");
	AddExhaust (th_main[0], 0.5, 0.025,(tex_main));
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new doge1 (hvessel, flightmodel);
}




// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (doge1*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void doge1::clbkPostCreation ()
{
	
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	ReplaceStockSound(MyID,"Sound\\Vessel\\attsustain.wav",	REPLACE_MAIN_THRUST);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);

	////////////////////////////////////

	
}
void doge1::clbkSaveState (FILEHANDLE scn)
{

	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters


	sprintf (cbuf, "%d", kilrot_sw);
	oapiWriteScenario_string (scn, "KROT", cbuf);


}
void doge1::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		
		if (!strnicmp (line, "KROT", 4))
		{
            sscanf (line+4, "%d", &kilrot_sw);
		}
		else
		{
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
	
}






int doge1::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{


  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) // +shift
	{
	 /* switch (key) 
	  {
		
	  }*/
	  return 0;
	} 
  else if (KEYMOD_CONTROL (kstate)) // +ctrl
	{
	return 0;
	}
  else // unmodified keys
	{ 
    switch (key) 
		{
		
		case OAPI_KEY_K: // photo
		if (kilrot_sw  == 0)kilrot_sw = 1;
		else kilrot_sw = 0;
		return 1;

		}
	return 0;
	}
  
 return 0;
 }


int doge1::clbkConsumeDirectKey(char *kstate)
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*2*dt;

if (KEYMOD_SHIFT (kstate))
{
if (KEYDOWN (kstate, OAPI_KEY_NUMPAD2))
		{
		if (avel.x < RAD*10)
			{
			avel.x = avel.x+drot;
			if (avel.x > RAD*10) avel.x = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD8))
		{
		if (avel.x > -RAD*10)
			{
			avel.x = avel.x-drot;
			if (avel.x < -RAD*10) avel.x = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD1))
		{
		if (avel.y < RAD*10)
			{
			avel.y = avel.y+drot;
			if (avel.y > RAD*10) avel.y = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD3))
		{
		if (avel.y > -RAD*10)
			{
			avel.y = avel.y-drot;
			if (avel.y < -RAD*10) avel.y = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD6))
		{
		if (avel.z < RAD*10)
			{			
			avel.z = avel.z+drot;
			if (avel.z > RAD*10) avel.z = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD4))
		{
		if (avel.z > -RAD*10)
			{
			avel.z = avel.z-drot;
			if (avel.z < -RAD*10) avel.z = -RAD*10;
			}		
		}

		SetAngularVel(avel);
	}
else // unmodified keys
	{
	
	}
return 0;
}





bool doge1::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{

	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	//int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	//int w2 = (int)(216*d);
	//int w3 = (int)(232*d);
	//int w4 = (int)(248*d);
	//int w5 = (int)(264*d);
	//int w6 = (int)(280*d);
	//int w7 = (int)(296*d);
	//int w8 = (int)(312*d);
	//int w9 = (int)(328*d);
	//int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);

	{
	skp->SetTextColor(0x0066FF66);
	double m1 = GetEmptyMass();
	double m2 = GetPropellantMass(hProp);
	double mass_fraction = ((m1+m2)/(m1));
	double dv = (MAIN_ISP*(log(mass_fraction)));
	char cbuf[256];
	sprintf(cbuf,"Available dV %0.1f m/s", dv);
	skp->Text ((10+roxl), (wc+royl), cbuf,strlen(cbuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Shift+NumPad = Reaction Wheel Attitude Control");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	if(kilrot_sw == 1)skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Shift+NumPad5 = Reaction Wheel KillRot");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}


	

	return true;
}


void doge1::clbkPreStep(double SimT, double SimDT, double MJD)
{
	
	if (kilrot_sw  == 1)DoKillRot();

	
}


void doge1::DoKillRot()
{
		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*2*dt;

		if (drot > abs(avel.x)) avel.x = 0;
		else avel.x = avel.x + (drot * ((abs(avel.x))/-avel.x));
		if (drot > abs(avel.y)) avel.y = 0;
		else avel.y = avel.y + (drot * ((abs(avel.y))/-avel.y));
		if (drot > abs(avel.z)) avel.z = 0;
		else avel.z = avel.z + (drot * ((abs(avel.z))/-avel.z));

		SetAngularVel (avel);
		if (avel.x == 0 && avel.y == 0 && avel.z == 0) kilrot_sw = 0;
}
