// ==============================================================
//                 ORBITER MODULE: nova_c
//                  
//
// nova_c.cpp
// Control module for nova_c vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include <math.h>
#include "OrbiterSoundSDK40.h"


// ==============================================================
// Some vessel parameters
// ==============================================================

const double EMP_MASS = 624;
const double FUEL_MASS = 1284;

const double RCSISP = 800;
const double RCSTH = 10;

const double MAINISP = 3600;
const double MAINTH = 4000;


const double LID_SPEED = 1;

double tgt_lat = 0;
double tgt_lng = 0;
int data_sw = 1;

VECTOR3 lidar_pos[3] = {
	lidar_pos[0] = _V(-0.523, 0.682, 0.054),
	lidar_pos[1] = _V(-0.673, 0.672, 0.100),
	lidar_pos[2] = _V(-0.684, 0.522, 0.054) 
	};

VECTOR3 lidar_dir[3] = {
	lidar_dir[0] = (_V(-0.540, 0.576, 0.156) - _V(-0.536, 0.609, 0.109)),
	lidar_dir[1] = (_V(-0.590, 0.589, 0.189) - _V(-0.617, 0.617, 0.146)),
	lidar_dir[2] = (_V(-0.620, 0.525, 0.189) - _V(-0.654, 0.520, 0.142)) 
	};

#define YELLOW RGB(255, 255, 80)
#define GREEN RGB(0, 225, 0)

double stiffness = 3e4;// 6e4
double damping = 1e4; // 5e4

static TOUCHDOWNVTX tdvtx_geardown[8] = {
	{_V( 0,-2, -2.12),     stiffness, damping, 0.9, 0.2},
	{_V(-1, 1.732, -2.12), stiffness, damping, 0.9, 0.2},
	{_V( 1, 1.732, -2.12), stiffness, damping, 0.9, 0.2},
	{_V( 0, 2, -2.12),     stiffness, damping, 0.9},
	{_V( 1,-1.732, -2.12), stiffness, damping, 0.9},
	{_V(-1,-1.732, -2.12), stiffness, damping, 0.9},
	{_V(0,0, 2.25),        stiffness, damping, 0.9}
};



class nova_c: public VESSEL3 {
public:
	
	nova_c (OBJHANDLE hVessel, int flightmodel);
	~nova_c ();

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	bool clbkLoadGenericCockpit ();
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//int clbkConsumeDirectKey(char *kstate);
	

private:
	void DefineAnimations (void);
	void SetCamDir(void);
	void GoToFlightVector(void);
	void DoDescent(void);
	void KillRCS(void);
	void GetTgtData(void);
	void DoTgtSearch(void);


	MGROUP_TRANSFORM *MgrpTran[1];
	UINT anim_lid;
	int MyID;
	THRUSTER_HANDLE th_rcs[17], th_group[4], th_dust;
	PROPELLANT_HANDLE hProp;
	//ANIMATIONCOMPONENT_HANDLE Section0, Section1;
	ATTACHMENTHANDLE hAttach_F9, hAttach_EC;
	double lid_proc, lidar_level, lidar_timer;
	int lid_status, lidar_status, view_status, ap_status, ap_mode;
	VECTOR3 flight_vector, y_vector;
	double step_time, tgt_elev;
	OBJHANDLE hMoon;
	double moon_rad, slope;
	VECTOR3 new_tgt_vtx_abs[12];
	int search_phase, search_status;
	double sel_tgt_lat, sel_tgt_lng, sel_tgt_slope, sel_tgt_elev;
	VECTOR3 tgt_vect_loc;
	double tgt_cross_range_error;
	double tgt_range;
	int pdi_sw, autosearch_sw;
	double mu;
	double col_timer;
	int col_sw;
};

nova_c::nova_c (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{

}

nova_c::~nova_c ()
{
	delete MgrpTran[0];
}

// Define animation sequences for moving parts
// --------------------------------------------------------------
void nova_c::DefineAnimations (void)
{
	// arm
	anim_lid = CreateAnimation (0);

	static UINT ArrayGrp0[1] = {16};
	MgrpTran[0] = new MGROUP_ROTATE(0, ArrayGrp0, 1, _V(0.579,0.780,1.436), _V(-0.7071,0.7071,0), (float)(PI));
	AddAnimationComponent (anim_lid, 0.0, 1.0, MgrpTran[0]);


}



// ==============================================================
// nova_c class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void nova_c::clbkSetClassCaps (FILEHANDLE cfg)
{
	lid_proc = 0.0;
	lid_status = 0;
	lidar_status = 0;
	lidar_level = 0.0;
	lidar_timer = 0.0;
	view_status = 0;
	ap_status = 0;
	ap_mode = 0;
	y_vector = _V(0,1,0);
	flight_vector = _V(0,0,1);
	step_time = 0;
	hMoon = NULL;
	tgt_elev = 0;
	moon_rad = 0;
	slope = 0;
	new_tgt_vtx_abs[0] = _V(0,0,0);
	new_tgt_vtx_abs[1] = _V(0,0,0);
	new_tgt_vtx_abs[2] = _V(0,0,0);
	new_tgt_vtx_abs[3] = _V(0,0,0);
	new_tgt_vtx_abs[4] = _V(0,0,0);
	new_tgt_vtx_abs[5] = _V(0,0,0);
	new_tgt_vtx_abs[6] = _V(0,0,0);
	new_tgt_vtx_abs[7] = _V(0,0,0);
	new_tgt_vtx_abs[8] = _V(0,0,0);
	new_tgt_vtx_abs[9] = _V(0,0,0);
	new_tgt_vtx_abs[10] = _V(0,0,0);
	new_tgt_vtx_abs[12] = _V(0,0,0);

	search_phase = 0;
	search_status = 0;
	sel_tgt_lat = 0;
	sel_tgt_lng = 0;
	sel_tgt_slope = 0;
	sel_tgt_elev = 0;
	
	tgt_vect_loc = _V(0,0,0);
	tgt_cross_range_error = 0;
	tgt_range = 0;
	pdi_sw = 0;
	mu = 0;
	autosearch_sw = 0;
	col_sw = 0;
	col_timer = 0.0;
	DefineAnimations ();

	// physical specs
	SetCW (1, 1, 1, 1);
	SetRotDrag (_V(1,1,1));
	SetPMI (_V(1.5,1.5,1));
	SetCrossSections (_V(7.3,7.3,4.8));
	SetSize (2.5);
	SetEmptyMass (EMP_MASS);
	SetMeshVisibilityMode ((AddMesh ("nova_c/nova_c")), MESHVIS_EXTERNAL);
	
	SetCameraOffset (_V(0,0,0));
	//SetTouchdownPoints (_V(0.1,0.8,-1.8), _V(0.512,-0.853,-1.8), _V(-0.512,-0.853,-1.8));
	SetTouchdownPoints ( tdvtx_geardown, 8);
	//SetDockParams (_V(0,0,-2), _V(0,0,-1), _V(0,1,0));

	// propellant resources

	hProp = CreatePropellantResource (FUEL_MASS, FUEL_MASS);

	
	// ***************** thruster definitions *******************

	th_rcs[0] = CreateThruster (_V( 0, 1, 1.82), _V( 1, 0, 0), RCSTH, hProp, RCSISP);
	th_rcs[1] = CreateThruster (_V( 0, 1, 1.82), _V(-1, 0, 0), RCSTH, hProp, RCSISP);
	th_rcs[2] = CreateThruster (_V( 0, 1, 1.82), _V( 0, 0, 1), RCSTH, hProp, RCSISP);
	th_rcs[3] = CreateThruster (_V( 0, 1, 1.82), _V( 0, 0,-1), RCSTH, hProp, RCSISP);

	th_rcs[4] = CreateThruster (_V( 0,-1, 1.82), _V( 1, 0, 0), RCSTH, hProp, RCSISP);
	th_rcs[5] = CreateThruster (_V( 0,-1, 1.82), _V(-1, 0, 0), RCSTH, hProp, RCSISP);
	th_rcs[6] = CreateThruster (_V( 0,-1, 1.82), _V( 0, 0, 1), RCSTH, hProp, RCSISP);
	th_rcs[7] = CreateThruster (_V( 0,-1, 1.82), _V( 0, 0,-1), RCSTH, hProp, RCSISP);

	th_rcs[8] = CreateThruster (_V( 1, 0, 1.82), _V( 0, 1, 0), RCSTH, hProp, RCSISP);
	th_rcs[9] = CreateThruster (_V( 1, 0, 1.82), _V( 0,-1, 0), RCSTH, hProp, RCSISP);
	th_rcs[10] = CreateThruster (_V( 1, 0, 1.82), _V( 0, 0, 1), RCSTH, hProp, RCSISP);
	th_rcs[11] = CreateThruster (_V( 1, 0, 1.82), _V( 0, 0,-1), RCSTH, hProp, RCSISP);

	th_rcs[12] = CreateThruster (_V(-1, 0, 1.82), _V( 0, 1, 0), RCSTH, hProp, RCSISP);
	th_rcs[13] = CreateThruster (_V(-1, 0, 1.82), _V( 0,-1, 0), RCSTH, hProp, RCSISP);
	th_rcs[14] = CreateThruster (_V(-1, 0, 1.82), _V( 0, 0, 1), RCSTH, hProp, RCSISP);
	th_rcs[15] = CreateThruster (_V(-1, 0, 1.82), _V( 0, 0,-1), RCSTH, hProp, RCSISP);

	th_rcs[16] = CreateThruster (_V( 0, 0, -2.0), _V(0,0,1), MAINTH, hProp, MAINISP);

	

	th_group[0] = th_rcs[ 8];
	th_group[1] = th_rcs[12];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[ 9];
	th_group[1] = th_rcs[13];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);
 
	th_group[0] = th_rcs[ 1];
	th_group[1] = th_rcs[ 5];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[ 0];
	th_group[1] = th_rcs[ 4];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[ 0];
	th_group[1] = th_rcs[ 5];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[ 1];
	th_group[1] = th_rcs[ 4];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[ 2];
	th_group[1] = th_rcs[ 6];
	th_group[2] = th_rcs[10];
	th_group[3] = th_rcs[14];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_FORWARD);

	th_group[0] = th_rcs[ 3];
	th_group[1] = th_rcs[ 7];
	th_group[2] = th_rcs[11];
	th_group[3] = th_rcs[15];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_BACK);

	th_group[0] = th_rcs[16];
	CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	///////////////////////////////////////

	SURFHANDLE tex_rcs = oapiRegisterExhaustTexture ("exhaust_atrcs");
	
	double ex_lng = 1.2;
	double ex_wdt = 0.06;

	VECTOR3 rcspos[16];
	rcspos[0] = _V(-0.069,0.965,1.809);
	rcspos[1] = _V( 0.069,0.965,1.827);
	rcspos[2] = _V(-0.011,0.965,1.751);
	rcspos[3] = _V( 0.011,0.965,1.884);

	rcspos[4] = _V(-0.069,-0.965,1.827);
	rcspos[5] = _V( 0.069,-0.965,1.809);
	rcspos[6] = _V(-0.011,-0.965,1.751);
	rcspos[7] = _V( 0.011,-0.965,1.884);

	rcspos[8] = _V(0.965,-0.069,1.809);
	rcspos[9] = _V(0.965, 0.069,1.827);
	rcspos[10] = _V(0.965,-0.011,1.751);
	rcspos[11] = _V(0.965, 0.011,1.884);

	rcspos[12] = _V(-0.965,-0.069,1.827);
	rcspos[13] = _V(-0.965, 0.069,1.809);
	rcspos[14] = _V(-0.965,-0.011,1.751);
	rcspos[15] = _V(-0.965, 0.011,1.884);

	VECTOR3 rcsdir[16];

	rcsdir[0] = _V(-1,0,0);
	rcsdir[1] = _V( 1,0,0);
	rcsdir[2] = _V(0,0,-1);
	rcsdir[3] = _V(0,0, 1);

	rcsdir[4] = _V(-1,0,0);
	rcsdir[5] = _V( 1,0,0);
	rcsdir[6] = _V(0,0,-1);
	rcsdir[7] = _V(0,0, 1);

	rcsdir[8] = _V(0,-1,0);
	rcsdir[9] = _V(0, 1,0);
	rcsdir[10] = _V(0,0,-1);
	rcsdir[11] = _V(0,0, 1);

	rcsdir[12] = _V(0,-1,0);
	rcsdir[13] = _V(0, 1,0);
	rcsdir[14] = _V(0,0,-1);
	rcsdir[15] = _V(0,0, 1);

	int i;
	for (i = 0; i < 16; i++)
	{
	//AddExhaust (th_rcs[i], ex_lng, ex_wdt, (tex_rcs));
	AddExhaust (th_rcs[i], ex_lng, ex_wdt, rcspos[i], rcsdir[i], (tex_rcs));
	}

	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("exhaust2");
	AddExhaust (th_rcs[16], 3, 0.3,(tex_main));

	///////////////////////// lidar
	SURFHANDLE lidar_ex = oapiRegisterExhaustTexture ("nova_c/lidar");

	double lidlen = 50;
	double lidwid = 0.02;

	EXHAUSTSPEC lidar_exhaust[3];

	

	normalise(lidar_dir[0]);
	normalise(lidar_dir[1]);
	normalise(lidar_dir[2]);

	//lidar_dir[0] = _V(0,0,1);

	for (int j = 0; j < 3; j++)
	{
	lidar_exhaust[j].th = NULL;
	lidar_exhaust[j].level = &lidar_level;
	lidar_exhaust[j].lpos = &lidar_pos[j];
	lidar_exhaust[j].ldir = &lidar_dir[j];
	lidar_exhaust[j].lsize = lidlen;
	lidar_exhaust[j].wsize = lidwid;
	lidar_exhaust[j].lofs = 0;
	lidar_exhaust[j].modulate = 0;
	lidar_exhaust[j].tex = lidar_ex;
	lidar_exhaust[j].flags = EXHAUST_CONSTANTPOS;
	lidar_exhaust[j].id = 0;
	AddExhaust (&lidar_exhaust[j]);
	}

	///////////////////////////////////////////////////////
	/////AirFoils

//	CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 0, 1, 1, 1);
//	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0, 1, 1, 1);

	//////////////////// attach //////////////////////////

	hAttach_F9 = CreateAttachment (true, _V( 0, 0,-1.182), _V( 0,0,-1), _V(0,1,0), "F9");
	hAttach_EC = CreateAttachment (false, _V( 0.457, 0.768, 1.523), _V( 0.7071, 0.7071,0), _V(0,0,1), "EC");

	/////////////////////////////// landing / takeoff dust

	th_dust = CreateThruster (_V(0, 0, -7), _V(0, 0, 1), 0, hProp, 1000000);

		SURFHANDLE exhaustdust = oapiRegisterParticleTexture ("Contrail1");

		PARTICLESTREAMSPEC dust_pstream = {
		0, 1, 20, 20, 0.5, 0.5, 10, 6, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1,
		exhaustdust
	};

	AddExhaustStream(th_dust, &dust_pstream);
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new nova_c (hvessel, flightmodel);
}




// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (nova_c*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void nova_c::clbkPostCreation ()
{
	
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	ReplaceStockSound(MyID,"Sound\\Vessel\\attsustain.wav",	REPLACE_MAIN_THRUST);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);

	/////

	hMoon = oapiGetGbodyByName("Moon");
	moon_rad = oapiGetSize(hMoon);
	double moon_mass = oapiGetMass(hMoon);
	mu = moon_mass*GGRAV;
	
}
void nova_c::clbkSaveState (FILEHANDLE scn)
{

	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters

	sprintf (cbuf, "%d %0.4f", lid_status, lid_proc);
	oapiWriteScenario_string (scn, "LID", cbuf);

	sprintf (cbuf, "%d", lidar_status);
	oapiWriteScenario_string (scn, "LASR", cbuf);

	sprintf (cbuf, "%d", view_status);
	oapiWriteScenario_string (scn, "VIEW", cbuf);

	sprintf (cbuf, "%d %d %d", ap_status, pdi_sw, autosearch_sw);
	oapiWriteScenario_string (scn, "AUTP", cbuf);

	sprintf (cbuf, "%0.5f %0.5f", tgt_lat, tgt_lng);
	oapiWriteScenario_string (scn, "TRGT", cbuf);

	//sprintf (cbuf, "%d", kilrot_sw);
	//oapiWriteScenario_string (scn, "KROT", cbuf);

	
	
}
void nova_c::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "LID", 3))
		{
            sscanf (line+3, "%d%lf", &lid_status, &lid_proc);
		}
		else if (!strnicmp (line, "LASR", 4))
		{
            sscanf (line+4, "%d", &lidar_status);
		}
		else if (!strnicmp (line, "VIEW", 4))
		{
            sscanf (line+4, "%d", &view_status);
		}
		else if (!strnicmp (line, "AUTP", 4))
		{
            sscanf (line+4, "%d%d%d", &ap_status, &pdi_sw, &autosearch_sw);
		}
		else if (!strnicmp (line, "TRGT", 4))
		{
            sscanf (line+4, "%lf%lf", &tgt_lat, &tgt_lng);
		}
		/*else if (!strnicmp (line, "KROT", 4))
		{
            sscanf (line+4, "%d", &kilrot_sw);
		}*/
		else
		{
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
	SetAnimation (anim_lid, lid_proc);
	//GetTgtData();
	
	
}


bool SetTgt(void *id, char *str, void *data)
{
	double new_tgt_lat = 0;
	double new_tgt_lng = 0;
//	char cbuf[256];
	//sscanf(str, "%s", &cbuf);
	sscanf (str, "%lf%lf", &new_tgt_lng, &new_tgt_lat);
	if(abs(new_tgt_lat) < 90 && abs(new_tgt_lng) < 360)
	{
		if(new_tgt_lng < -180)new_tgt_lng = new_tgt_lng + 360;
		else if(new_tgt_lng > 180)new_tgt_lng = new_tgt_lng - 360;
		tgt_lat = new_tgt_lat;
		tgt_lng = new_tgt_lng;
		data_sw = 1;
		return true;
	}
	else return false;
}



int nova_c::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{

//bool cbSelectTarget(void *id, char *str, void *data);

  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) // +shift
	{
	  /*switch (key) 
	  {
		case OAPI_KEY_NUMPAD5: 
		if (kilrot_sw  == 0)kilrot_sw = 1;
		else kilrot_sw = 0;
		return 1;
	  }*/
	  return 0;
	} 
  else if (KEYMOD_CONTROL (kstate)) // +ctrl
	{
	return 0;
	}
  else // unmodified keys
	{ 
    switch (key) 
		{
		case OAPI_KEY_V:
		oapiOpenInputBox("Enter Target Lng. Lat.",SetTgt, 0, 50, 0);
		return 1;

		case OAPI_KEY_P: // descent autopilot
		if (ap_status  == 0)ap_status = 1;
		else 
			{
			ap_status = 0;
			KillRCS();
			}
		return 1;

		case OAPI_KEY_J: // Lid
		if (lid_status  == 0)lid_status = 1;
		return 1;

		case OAPI_KEY_K: // Lidar
		if (lidar_status  == 0)lidar_status = 1;
		else 
			{
			lidar_status = 0;
			lidar_level = 0;
			}
		return 1;

		case OAPI_KEY_E: // view forward back
		if (view_status  == 0)view_status = 1;
		else view_status = 0;
		SetCamDir();
		return 1;

		case OAPI_KEY_B: // do search
		if (search_status  == 0)search_status = 1;
		else 
			{
			search_status = 0;
			search_phase = 0;
			}
		return 1;

		case OAPI_KEY_N: // do search
			{
			tgt_lat = sel_tgt_lat*DEG;
			tgt_lng = sel_tgt_lng*DEG;
			GetTgtData();
			}
		return 1;
		}
	return 0;
	}
  
 return 0;
 }



bool nova_c::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{

	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	//int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);

	{
	skp->SetTextColor(0x0066FF66);
	double m1 = GetEmptyMass();
	double m2 = GetPropellantMass(hProp);
	double mass_fraction = ((m1+m2)/(m1));
	double dv = (MAINISP*(log(mass_fraction)));
	char cbuf[256];
	sprintf(cbuf,"Available dV %0.1f m/s", dv);
	skp->Text ((10+roxl), (wc+royl), cbuf,strlen(cbuf));
	}

	

	
	
	

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"V = Enter Landing Target Lng. Lat.", tgt_lng, tgt_lat);
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Target Lng Lat  %0.4f  %0.4f",tgt_lng, tgt_lat);
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Target Range %0.3fkm",tgt_range*0.001);
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Crossrange  %0.3fkm",-tgt_cross_range_error*0.001);
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Target Elev: %0.0fm", tgt_elev);
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Target Slope: %0.1f", (slope*DEG));
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	if(search_status == 0)sprintf(abuf,"B = Do Landing Site Search");
	else 
		{
		skp->SetTextColor(0x0066FFFF);
		sprintf(abuf,"SEARCHING");
		}
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}

	{
	//skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Suggested Lng: %0.4f Lat: %0.4f Elev: %0.0fm Slope: %0.1f", sel_tgt_lng*DEG, sel_tgt_lat*DEG, sel_tgt_elev, (sel_tgt_slope*DEG));
	skp->Text ((10+roxl), (w9+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"N = Use Suggested Landing Lng/Lat");
	skp->Text ((10+roxl), (w10+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	if(ap_status == 1)skp->SetTextColor(0x0066FFFF);
	else if(col_sw == 1) skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"P = AUTO DESCENT:ON/OFF");
	skp->Text ((10+roxl), (w12+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	if(view_status == 0)sprintf(abuf,"E = View:FORWARD");
	else sprintf(abuf,"E = View:REAR");
	skp->Text ((10+roxl), (w13+royl), abuf,strlen(abuf));
	}


	if(lid_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Deploy EagleCam");
	skp->Text ((10+roxl), (w14+royl), abuf,strlen(abuf));
	}

	
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	if(lidar_status == 1)skp->SetTextColor(0x0066FFFF);
	sprintf(abuf,"K = Lidar:ON/OFF");
	skp->Text ((10+roxl), (w15+royl), abuf,strlen(abuf));
	}


	return true;
}

void nova_c::KillRCS(void)
{
	SetThrusterGroupLevel(THGROUP_ATT_PITCHUP,0);
	SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN,0);
	SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT,0);
	SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT,0);
	SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT,0);
	SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT,0);
}


void nova_c::clbkPreStep(double SimT, double SimDT, double MJD)
{
	step_time = SimDT;

	////////////// ap ready warning
	VECTOR3 relpos;
	GetRelativePos(hMoon,relpos);
	if((length(relpos)-moon_rad) < 150000)
	{
		if(tgt_range < 1000000 && tgt_range > 650000)
		{
			col_timer = col_timer + SimDT;
			if(col_timer > 1)
			{
			if(col_sw == 0)col_sw = 1;
			else col_sw = 0;
			col_timer = 0;
			}
		}
		else col_sw = 0;
	}
	else col_sw = 0;
	////////////////////////////////////////

	if(data_sw == 1)
	{
		GetTgtData();
		data_sw = 0;
	}

	if(search_status > 0)DoTgtSearch();
	
	DoDescent();

	///////// animations //////////////////////////////////

	// deploy lid + ec
	if (lid_status == 0) lid_proc = 0;
	else if (lid_status == 1) 
	{
		double da = SimDT * LID_SPEED;
		lid_proc = lid_proc + da;
		if (lid_proc > 1) 
		{
			lid_proc = 1;
			lid_status = 2;
			DetachChild(hAttach_EC, 1);
		}
	}
	else if (lid_status == 2) lid_proc = 1;
	SetAnimation (anim_lid, lid_proc);

	//////////////////// lidar flash
//	sprintf(oapiDebugString(), "lidar_status %d", lidar_status);
	if (lidar_status  == 1)
	{
		lidar_timer = lidar_timer + SimDT;
		if(lidar_timer < 0.05)lidar_level = 1;
		else if(lidar_timer < 0.1)lidar_level = 0;
		else lidar_timer = 0;	
	}
	else 
	{
		lidar_level = 0;
		lidar_timer = 0;
	}

	////////////////// landing dust
	double dust_level = 0;
	double altdust = GetAltitude(ALTMODE_GROUND);
	if(altdust < 7)
	{
	double sin_pitch = sin(GetPitch());
	if(sin_pitch > 0.7) 
		{
		altdust = altdust/sin_pitch;
		dust_level = GetThrusterGroupLevel(THGROUP_MAIN);
		SetThrusterRef(th_dust, _V(0,0,-altdust));
		}
	else dust_level = 0;
	}
	else dust_level = 0;
	SetThrusterLevel(th_dust, dust_level);
	
}

bool nova_c::clbkLoadGenericCockpit ()
{
	SetCamDir();
return true;
}

void nova_c::SetCamDir()
{
	VECTOR3 camdefdir = _V(0,0,1);
	if(view_status == 1)camdefdir = _V(0,0,-1);
	SetCameraDefaultDirection(camdefdir, 0);
	oapiCameraAttach(GetHandle(), 2);
}

void nova_c::GetTgtData()
{
	
	tgt_elev = oapiSurfaceElevation(hMoon, (tgt_lng*RAD), (tgt_lat*RAD));
//	tgt_vect_loc;
	oapiEquToLocal(hMoon,(tgt_lng*RAD),(tgt_lat*RAD),(tgt_elev+moon_rad), &tgt_vect_loc);
	VECTOR3 tgt_vect_loc_nm = tgt_vect_loc;
	normalise(tgt_vect_loc_nm);

	VECTOR3 rot_axis_nml = crossp(_V(0,1,0), tgt_vect_loc);
	normalise(rot_axis_nml);
	double rot_angle = 0.000066*RAD;
	MATRIX3 rm = rotm(rot_axis_nml, rot_angle);

	VECTOR3 triangle_vtx[3];
	VECTOR3 triangle_vtx_nm[3];
//	double triangle_elev[3];
	double triangle_lat[3];
	double triangle_lng[3];
	triangle_vtx[0] = mul(rm,tgt_vect_loc); 
	rm = rotm(tgt_vect_loc_nm, (120*RAD));
	triangle_vtx[1] = mul(rm, triangle_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (240*RAD));
	triangle_vtx[2] = mul(rm, triangle_vtx[0]);

	double rad;
	oapiLocalToEqu (hMoon, triangle_vtx[0], &triangle_lng[0] , &triangle_lat[0], &rad);
	oapiLocalToEqu (hMoon, triangle_vtx[1], &triangle_lng[1] , &triangle_lat[1], &rad);
	oapiLocalToEqu (hMoon, triangle_vtx[2], &triangle_lng[2] , &triangle_lat[2], &rad);

	double triangle_elev0 = oapiSurfaceElevation(hMoon, triangle_lng[0], triangle_lat[0]);
	double triangle_elev1 = oapiSurfaceElevation(hMoon, triangle_lng[1], triangle_lat[1]);
	double triangle_elev2 = oapiSurfaceElevation(hMoon, triangle_lng[2], triangle_lat[2]);

	triangle_vtx_nm[0] = triangle_vtx[0];
	normalise(triangle_vtx_nm[0]);
	triangle_vtx_nm[0].x = triangle_vtx_nm[0].x * (triangle_elev0 + moon_rad);
	triangle_vtx_nm[0].y = triangle_vtx_nm[0].y * (triangle_elev0 + moon_rad);
	triangle_vtx_nm[0].z = triangle_vtx_nm[0].z * (triangle_elev0 + moon_rad);

	triangle_vtx_nm[1] = triangle_vtx[1];
	normalise(triangle_vtx_nm[1]);
	triangle_vtx_nm[1].x = triangle_vtx_nm[1].x * (triangle_elev1 + moon_rad);
	triangle_vtx_nm[1].y = triangle_vtx_nm[1].y * (triangle_elev1 + moon_rad);
	triangle_vtx_nm[1].z = triangle_vtx_nm[1].z * (triangle_elev1 + moon_rad);

	triangle_vtx_nm[2] = triangle_vtx[2];
	normalise(triangle_vtx_nm[2]);
	triangle_vtx_nm[2].x = triangle_vtx_nm[2].x * (triangle_elev2 + moon_rad);
	triangle_vtx_nm[2].y = triangle_vtx_nm[2].y * (triangle_elev2 + moon_rad);
	triangle_vtx_nm[2].z = triangle_vtx_nm[2].z * (triangle_elev2 + moon_rad);

	VECTOR3 surface_vect[2];
	surface_vect[0] = triangle_vtx_nm[1] - triangle_vtx_nm[0];
	surface_vect[1] = triangle_vtx_nm[2] - triangle_vtx_nm[0];

	VECTOR3 surf_nml = crossp(surface_vect[0], surface_vect[1]);
	normalise(surf_nml);
	slope = acos(dotp(tgt_vect_loc_nm,surf_nml));



}

void nova_c::DoTgtSearch()
{
	if(search_phase == 0)
	{
	if(search_status == 1)
		{
		sel_tgt_elev = tgt_elev;
		sel_tgt_lat = tgt_lat*RAD;
		sel_tgt_lng = tgt_lng*RAD;
		sel_tgt_slope = slope;
		}

	double new_tgt_elev = oapiSurfaceElevation(hMoon, (tgt_lng*RAD), (tgt_lat*RAD));
//	VECTOR3 tgt_vect_loc;
//	oapiEquToLocal(hMoon,(tgt_lng*RAD),(tgt_lat*RAD),(new_tgt_elev+moon_rad), &tgt_vect_loc);
	VECTOR3 tgt_vect_loc_nm = tgt_vect_loc;
	normalise(tgt_vect_loc_nm);

	VECTOR3 rot_axis_nml = crossp(_V(0,1,0), tgt_vect_loc);
	normalise(rot_axis_nml);
	double rot_angle = 0.00066*RAD*search_status;
	MATRIX3 rm = rotm(rot_axis_nml, rot_angle);

	VECTOR3 new_tgt_vtx[12];
	VECTOR3 new_tgt_vtx_nm[12];
//	double new_tgt_elev[6];
	double new_tgt_lat[12];
	double new_tgt_lng[12];

	new_tgt_vtx[0] = mul(rm,tgt_vect_loc); 
	rm = rotm(tgt_vect_loc_nm, (30*RAD));
	new_tgt_vtx[1] = mul(rm, new_tgt_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (60*RAD));
	new_tgt_vtx[2] = mul(rm, new_tgt_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (90*RAD));
	new_tgt_vtx[3] = mul(rm,new_tgt_vtx[0]); 
	rm = rotm(tgt_vect_loc_nm, (120*RAD));
	new_tgt_vtx[4] = mul(rm, new_tgt_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (160*RAD));
	new_tgt_vtx[5] = mul(rm, new_tgt_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (180*RAD));
	new_tgt_vtx[6] = mul(rm,tgt_vect_loc); 
	rm = rotm(tgt_vect_loc_nm, (210*RAD));
	new_tgt_vtx[7] = mul(rm, new_tgt_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (240*RAD));
	new_tgt_vtx[8] = mul(rm, new_tgt_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (270*RAD));
	new_tgt_vtx[9] = mul(rm,new_tgt_vtx[0]); 
	rm = rotm(tgt_vect_loc_nm, (300*RAD));
	new_tgt_vtx[10] = mul(rm, new_tgt_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (330*RAD));
	new_tgt_vtx[11] = mul(rm, new_tgt_vtx[0]);



	double rad;
	oapiLocalToEqu (hMoon, new_tgt_vtx[0], &new_tgt_lng[0] , &new_tgt_lat[0], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[1], &new_tgt_lng[1] , &new_tgt_lat[1], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[2], &new_tgt_lng[2] , &new_tgt_lat[2], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[3], &new_tgt_lng[3] , &new_tgt_lat[3], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[4], &new_tgt_lng[4] , &new_tgt_lat[4], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[5], &new_tgt_lng[5] , &new_tgt_lat[5], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[6], &new_tgt_lng[6] , &new_tgt_lat[6], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[7], &new_tgt_lng[7] , &new_tgt_lat[7], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[8], &new_tgt_lng[8] , &new_tgt_lat[8], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[9], &new_tgt_lng[9] , &new_tgt_lat[9], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[10], &new_tgt_lng[10] , &new_tgt_lat[10], &rad);
	oapiLocalToEqu (hMoon, new_tgt_vtx[11], &new_tgt_lng[11] , &new_tgt_lat[11], &rad);

	double new_tgt_elev0 = oapiSurfaceElevation(hMoon, new_tgt_lng[0], new_tgt_lat[0]);
	double new_tgt_elev1 = oapiSurfaceElevation(hMoon, new_tgt_lng[1], new_tgt_lat[1]);
	double new_tgt_elev2 = oapiSurfaceElevation(hMoon, new_tgt_lng[2], new_tgt_lat[2]);
	double new_tgt_elev3 = oapiSurfaceElevation(hMoon, new_tgt_lng[3], new_tgt_lat[3]);
	double new_tgt_elev4 = oapiSurfaceElevation(hMoon, new_tgt_lng[4], new_tgt_lat[4]);
	double new_tgt_elev5 = oapiSurfaceElevation(hMoon, new_tgt_lng[5], new_tgt_lat[5]);
	double new_tgt_elev6 = oapiSurfaceElevation(hMoon, new_tgt_lng[6], new_tgt_lat[6]);
	double new_tgt_elev7 = oapiSurfaceElevation(hMoon, new_tgt_lng[7], new_tgt_lat[7]);
	double new_tgt_elev8 = oapiSurfaceElevation(hMoon, new_tgt_lng[8], new_tgt_lat[8]);
	double new_tgt_elev9 = oapiSurfaceElevation(hMoon, new_tgt_lng[9], new_tgt_lat[9]);
	double new_tgt_elev10 = oapiSurfaceElevation(hMoon, new_tgt_lng[10], new_tgt_lat[10]);
	double new_tgt_elev11 = oapiSurfaceElevation(hMoon, new_tgt_lng[11], new_tgt_lat[11]);

	new_tgt_vtx_nm[0] = new_tgt_vtx[0];
	normalise(new_tgt_vtx_nm[0]);
	new_tgt_vtx_abs[0] = new_tgt_vtx_nm[0] * (new_tgt_elev0 + moon_rad);

	new_tgt_vtx_nm[1] = new_tgt_vtx[1];
	normalise(new_tgt_vtx_nm[1]);
	new_tgt_vtx_abs[1] = new_tgt_vtx_nm[1] * (new_tgt_elev1 + moon_rad);

	new_tgt_vtx_nm[2] = new_tgt_vtx[2];
	normalise(new_tgt_vtx_nm[2]);
	new_tgt_vtx_abs[2] = new_tgt_vtx_nm[2] * (new_tgt_elev2 + moon_rad);

	new_tgt_vtx_nm[3] = new_tgt_vtx[3];
	normalise(new_tgt_vtx_nm[3]);
	new_tgt_vtx_abs[3] = new_tgt_vtx_nm[3] * (new_tgt_elev3 + moon_rad);

	new_tgt_vtx_nm[4] = new_tgt_vtx[4];
	normalise(new_tgt_vtx_nm[4]);
	new_tgt_vtx_abs[4] = new_tgt_vtx_nm[4] * (new_tgt_elev4 + moon_rad);

	new_tgt_vtx_nm[5] = new_tgt_vtx[5];
	normalise(new_tgt_vtx_nm[5]);
	new_tgt_vtx_abs[5] = new_tgt_vtx_nm[5] * (new_tgt_elev5 + moon_rad);

	new_tgt_vtx_nm[6] = new_tgt_vtx[6];
	normalise(new_tgt_vtx_nm[6]);
	new_tgt_vtx_abs[6] = new_tgt_vtx_nm[6] * (new_tgt_elev6 + moon_rad);

	new_tgt_vtx_nm[7] = new_tgt_vtx[7];
	normalise(new_tgt_vtx_nm[7]);
	new_tgt_vtx_abs[7] = new_tgt_vtx_nm[7] * (new_tgt_elev7 + moon_rad);

	new_tgt_vtx_nm[8] = new_tgt_vtx[8];
	normalise(new_tgt_vtx_nm[8]);
	new_tgt_vtx_abs[8] = new_tgt_vtx_nm[8] * (new_tgt_elev8 + moon_rad);

	new_tgt_vtx_nm[9] = new_tgt_vtx[9];
	normalise(new_tgt_vtx_nm[9]);
	new_tgt_vtx_abs[9] = new_tgt_vtx_nm[9] * (new_tgt_elev9 + moon_rad);

	new_tgt_vtx_nm[10] = new_tgt_vtx[10];
	normalise(new_tgt_vtx_nm[10]);
	new_tgt_vtx_abs[10] = new_tgt_vtx_nm[10] * (new_tgt_elev10 + moon_rad);

	new_tgt_vtx_nm[11] = new_tgt_vtx[11];
	normalise(new_tgt_vtx_nm[11]);
	new_tgt_vtx_abs[11] = new_tgt_vtx_nm[11] * (new_tgt_elev11 + moon_rad);

	search_phase = 1;
	}

	
	else if (search_phase > 0)
	{
	int i = search_phase - 1;

	double new_tgt_elev = length(new_tgt_vtx_abs[i]) - moon_rad;
	VECTOR3 tgt_vect_loc1 = new_tgt_vtx_abs[i];
//	oapiEquToLocal(hMoon,(tgt_lng*RAD),(tgt_lat*RAD),tgt_elev, &tgt_vect_loc);
	VECTOR3 tgt_vect_loc_nm = tgt_vect_loc1;
	normalise(tgt_vect_loc_nm);

	VECTOR3 rot_axis_nml = crossp(_V(0,1,0), tgt_vect_loc1);
	normalise(rot_axis_nml);
	double rot_angle = 0.000066*RAD;
	MATRIX3 rm = rotm(rot_axis_nml, rot_angle);

	VECTOR3 triangle_vtx[3];
	VECTOR3 triangle_vtx_nm[3];
	//double triangle_elev[3];
	double triangle_lat[3];
	double triangle_lng[3];

	triangle_vtx[0] = mul(rm,tgt_vect_loc1); 
	rm = rotm(tgt_vect_loc_nm, (120*RAD));
	triangle_vtx[1] = mul(rm, triangle_vtx[0]);
	rm = rotm(tgt_vect_loc_nm, (240*RAD));
	triangle_vtx[2] = mul(rm, triangle_vtx[0]);

	double rad;
	oapiLocalToEqu (hMoon, triangle_vtx[0], &triangle_lng[0] , &triangle_lat[0], &rad);
	oapiLocalToEqu (hMoon, triangle_vtx[1], &triangle_lng[1] , &triangle_lat[1], &rad);
	oapiLocalToEqu (hMoon, triangle_vtx[2], &triangle_lng[2] , &triangle_lat[2], &rad);

	double triangle_elev0 = oapiSurfaceElevation(hMoon, triangle_lng[0], triangle_lat[0]);
	double triangle_elev1 = oapiSurfaceElevation(hMoon, triangle_lng[1], triangle_lat[1]);
	double triangle_elev2 = oapiSurfaceElevation(hMoon, triangle_lng[2], triangle_lat[2]);

	triangle_vtx_nm[0] = triangle_vtx[0];
	normalise(triangle_vtx_nm[0]);
	triangle_vtx_nm[0] = triangle_vtx_nm[0] * (triangle_elev0 + moon_rad);
	

	triangle_vtx_nm[1] = triangle_vtx[1];
	normalise(triangle_vtx_nm[1]);
	triangle_vtx_nm[1] = triangle_vtx_nm[1] * (triangle_elev1 + moon_rad);
	

	triangle_vtx_nm[2] = triangle_vtx[2];
	normalise(triangle_vtx_nm[2]);
	triangle_vtx_nm[2] = triangle_vtx_nm[2] * (triangle_elev2 + moon_rad);
	
	VECTOR3 surface_vect[2];
	surface_vect[0] = triangle_vtx_nm[1] - triangle_vtx_nm[0];
	surface_vect[1] = triangle_vtx_nm[2] - triangle_vtx_nm[0];

	VECTOR3 surf_nml = crossp(surface_vect[0], surface_vect[1]);
	normalise(surf_nml);
	double new_tgt_slope = acos(dotp(tgt_vect_loc_nm,surf_nml));
	if(new_tgt_slope < sel_tgt_slope)
		{
		sel_tgt_elev = new_tgt_elev;
		sel_tgt_slope = new_tgt_slope;
		oapiLocalToEqu (hMoon, tgt_vect_loc1, &sel_tgt_lng , &sel_tgt_lat, &rad);
		}
	search_phase = search_phase + 1;
	if(search_phase == 13)
		{
		search_status = search_status+1;
		if(search_status == 21)search_status = 0;
		search_phase = 0;
		}
	}

}

void nova_c::clbkPostStep (double simt, double simdt, double mjd)
{
	if(ap_status > 0)
	{
		if(GroundContact() == true)
		{
		ap_status = 0;
		KillRCS();
		SetThrusterGroupLevel(THGROUP_MAIN, 0);
		//lidar_status = 0;
		}
	}
}

void nova_c::GoToFlightVector()
{
DeactivateNavmode(NAVMODE_KILLROT);

//VECTOR3 haspdv;
//GetAirspeedVector(FRAME_LOCAL,haspdv);
//haspdv = -haspdv;
//normalise(haspdv);

//flight_vector = haspdv;

double vrot_lim = 5*RAD;
double mass = GetMass();
double max_torque_x = 20*1.82;
double max_torque_y = 20*1.82;
double max_torque_z = 20*1;
VECTOR3 pmi;
GetPMI(pmi);

double angoa = atan2(-flight_vector.y, flight_vector.z);
double slipage = atan2(flight_vector.x, flight_vector.z);

//sprintf(oapiDebugString(), "angoa %0.1f slipage %0.1f", angoa*DEG, slipage*DEG);

VECTOR3 up_local;
HorizonInvRot(_V(0,1,0),up_local);
VECTOR3 up_vel_nml = crossp(up_local, flight_vector);
normalise(up_vel_nml);
y_vector = up_vel_nml;

if((GetAltitude(ALTMODE_MEANRAD) - tgt_elev) < 10000)
{
	/*VECTOR3 sun_vector_local;
	Global2Local(_V(0,0,0),sun_vector_local);
	normalise(sun_vector_local);
	
	double angle_off = (-135*RAD);
	MATRIX3 rm2 = rotm(_V(0,0,1),angle_off);
	sun_vector_local = mul(rm2,sun_vector_local);
	y_vector = sun_vector_local;*/

	VECTOR3 sun_at_noon_hrz = _V(0,0,-1);
	if(tgt_lat < 0) sun_at_noon_hrz = _V(0,0,1);
	VECTOR3 sun_at_noon_loc;
	HorizonInvRot(sun_at_noon_hrz, sun_at_noon_loc);
	double angle_off = (-135*RAD);
	MATRIX3 rm2 = rotm(_V(0,0,1),angle_off);
	sun_at_noon_loc = mul(rm2,sun_at_noon_loc);
	y_vector = sun_at_noon_loc;
}


double roll_angle = atan2(-y_vector.x,y_vector.y);
//if(piv_pos_local.y < 0)roll_angle = atan2(-piv_pos_local.x,piv_pos_local.y);
if(roll_angle < -PI)roll_angle = roll_angle + PI2;
else if(roll_angle > PI)roll_angle = roll_angle - PI2;

if(angoa > PI)angoa = (PI*2)-angoa;
else if(angoa < -PI)angoa = (PI*2)+angoa;

if(slipage > PI)slipage = (PI*2)-slipage;
else if(slipage < -PI)slipage = (PI*2)+slipage;

VECTOR3 avel = _V(0,0,0);
GetAngularVel(avel);
double max_acc_x = max_torque_x/(pmi.x*mass);
double max_drot_x = max_acc_x*step_time;
double max_acc_y = max_torque_y/(pmi.y*mass);
double max_drot_y = max_acc_y*step_time;
double max_acc_z = max_torque_z/(pmi.z*mass);
double max_drot_z = max_acc_z*step_time;

double reqxv = 0.9*(sqrt(abs(angoa*max_acc_x*2)));
if (reqxv > vrot_lim)reqxv = vrot_lim;
if (angoa > 0)reqxv = -reqxv;
double req_drotx = reqxv - avel.x;
double thx_lvl = req_drotx/max_drot_x;
if((angoa*DEG) < 1 && (angoa*DEG) > -1)
{
reqxv = 1*-angoa;
double acc_reqx = (reqxv-avel.x)/0.2;
thx_lvl = acc_reqx/max_acc_x;
}

double reqyv = 0.9*(sqrt(abs(slipage*max_acc_y*2)));
if (reqyv > vrot_lim)reqyv = vrot_lim;
if (slipage > 0)reqyv = -reqyv;
double req_droty = reqyv - avel.y;
double thy_lvl = req_droty/max_drot_y;
if((slipage*DEG) < 1 && (slipage*DEG) > -1)
{
reqyv = 1*-slipage;
double acc_reqy = (reqyv-avel.y)/0.2;
thy_lvl = acc_reqy/max_acc_y;
}
//////////////////////////////////////////////
if(thx_lvl > 1)thx_lvl = 1;
else if(thx_lvl < -1)thx_lvl = -1;

if(thy_lvl > 1)thy_lvl = 1;
else if(thy_lvl < -1)thy_lvl = -1;


if(thx_lvl > 0)
{
SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, thx_lvl);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
}
else if(thx_lvl < 0)
{
SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, -thx_lvl);
}
else
{
SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
}


if(thy_lvl > 0)
{
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, thy_lvl);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
}
else if(thy_lvl < 0)
{
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, -thy_lvl);
}
else
{
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
}
//sprintf(oapiDebugString(), "thx_lvl %0.1f thy_lvl %0.1f", thx_lvl, thy_lvl);
//////////////////// roll

double roll_lim = 0.2*roll_angle;
if(roll_lim > (10*RAD))roll_lim = (10*RAD);
else if(roll_lim < (-10*RAD))roll_lim = (-10*RAD);

double avel_z_req = (-roll_lim) - avel.z;
double thz_lvl = avel_z_req/max_drot_z;
if(thz_lvl > 1)thz_lvl = 1;
else if(thz_lvl < -1)thz_lvl = -1;


///////////////////// th logic

double th0_lvl = 0;
double th5_lvl = 0; 
double th1_lvl = 0; 
double th4_lvl = 0;

if(thz_lvl > 0)
{
th0_lvl = thz_lvl;
th5_lvl = thz_lvl;
th1_lvl = 0;
th4_lvl = 0;
}
else if(thz_lvl < 0)
{
th0_lvl = 0;
th5_lvl = 0;
th1_lvl = -thz_lvl;
th4_lvl = -thz_lvl;
}
else
{
th0_lvl = 0;
th5_lvl = 0;
th1_lvl = 0;
th4_lvl = 0;
}
///////////////////////////////////
if(thy_lvl > 0)
{
th5_lvl = th5_lvl + thy_lvl; 
th1_lvl = th1_lvl + thy_lvl;  
}
else if(thy_lvl < 0)
{
th0_lvl = th0_lvl - thy_lvl; 
th4_lvl = th4_lvl - thy_lvl;  
}

SetThrusterLevel(th_rcs[1],th1_lvl);
SetThrusterLevel(th_rcs[4],th4_lvl);
SetThrusterLevel(th_rcs[0],th0_lvl);
SetThrusterLevel(th_rcs[5],th5_lvl);
//////////////////////////////////


}

void nova_c::DoDescent()
{

	if(GroundContact() == true)
	{
		ap_status = 0;
		KillRCS();
		SetThrusterGroupLevel(THGROUP_MAIN, 0);
		//lidar_status = 0;
		return;
	}


	double mass = GetMass();
	double th_max = MAINTH;
	double altmr = GetAltitude(ALTMODE_MEANRAD);

	


	VECTOR3 tgt_vector_glob;
	oapiEquToGlobal (hMoon, (tgt_lng*RAD), (tgt_lat*RAD), (tgt_elev+moon_rad), &tgt_vector_glob);
	VECTOR3 tgt_vector_ves;
	Global2Local(tgt_vector_glob, tgt_vector_ves);
	VECTOR3 tgt_vector_hrz;
	HorizonRot(tgt_vector_ves,tgt_vector_hrz);

	VECTOR3 gspdv_hrz;
	GetGroundspeedVector(FRAME_HORIZON, gspdv_hrz);

	VECTOR3 vel_rel_glob;
	GetRelativeVel(hMoon, vel_rel_glob);

	VECTOR3 pos_rel_glob;
	GetRelativePos(hMoon, pos_rel_glob);

	VECTOR3 tgt_pos_rel_glob;
	Local2Rel(tgt_vector_ves, tgt_pos_rel_glob);

	VECTOR3 orb_nml_glob = crossp(pos_rel_glob, vel_rel_glob);
	normalise(orb_nml_glob);

	VECTOR3 tgt_pos_rel_glob_nm = tgt_pos_rel_glob;
	normalise(tgt_pos_rel_glob_nm);
	
	double tgt_op_angle = PI05 - acos(dotp(orb_nml_glob,tgt_pos_rel_glob_nm));
	tgt_cross_range_error = tgt_op_angle * moon_rad;

	VECTOR3 ves_tgt_nml_glob_nm = crossp(pos_rel_glob, tgt_pos_rel_glob);
	normalise(ves_tgt_nml_glob_nm);

	double axis_angle = acos(dotp(orb_nml_glob, ves_tgt_nml_glob_nm));
	int app_dep = 0;
	if(axis_angle > PI05)app_dep = 1;

	VECTOR3 pos_rel_glob_nm = pos_rel_glob;
	normalise(pos_rel_glob_nm);
	double range_angle = acos(dotp(tgt_pos_rel_glob_nm, pos_rel_glob_nm));

	tgt_range = range_angle * moon_rad;

	if(app_dep == 1)
	{
	tgt_range = (PI2 - range_angle) * moon_rad;
	}

	if(ap_status == 1)
	{

	if((altmr - tgt_elev) < 5000 && autosearch_sw == 0)
		{
		if(search_status == 0)
			{
			search_status = 1;
			autosearch_sw = 1;
			lidar_status = 1;
			}
		}

	if(autosearch_sw == 1 && search_status == 0)
		{
		tgt_lat = sel_tgt_lat*DEG;
		tgt_lng = sel_tgt_lng*DEG;
		GetTgtData();
		autosearch_sw = 2; // finished
		lidar_status = 0;
		}
	
	
	if((altmr - tgt_elev) < 50 && lid_status == 0)lid_status = 1;
	

	//VECTOR3 g_vector_ves;
	//GetWeightVector(g_vector_ves);
	//VECTOR3 g_vector_hrz;
	//HorizonRot(g_vector_ves, g_vector_hrz);
	double r2 = (altmr+moon_rad)*(altmr+moon_rad);
	double g_acc = -mu/r2;

	double v2 = length(vel_rel_glob);
	v2 = v2*v2;
	double y2 = gspdv_hrz.y * gspdv_hrz.y;
	double o_speed_abs = sqrt(v2 - y2);

	double c_acc = (o_speed_abs*o_speed_abs)/(moon_rad+altmr);
	double net_g = g_acc + c_acc;
	double max_acc = th_max/mass;

	double gs2 = length(gspdv_hrz);
	gs2 = gs2 * gs2;
	double g_speed_abs = sqrt(gs2 - (gspdv_hrz.y * gspdv_hrz.y));

	//double wp_range = tgt_range - waypoint[wp].x;
	//double g_speed_wp = g_speed_abs-waypoint[wp].z;

	//double air_range = (moon_rad/(moon_rad+altmr))*tgt_range;

	double stop_time = (tgt_range-2000)/(0.5*(g_speed_abs+10));
	double acc_req_h = (g_speed_abs-10)/stop_time;
	double alt_net = -altmr + tgt_elev + 4500;
	double vvel_req = (2*(alt_net/stop_time))+100;
	double acc_req_v = (2*(vvel_req-gspdv_hrz.y)/(stop_time*0.5)) - net_g;
	//double acc_req_v = (-2*(alt_net- (stop_time*-100))/(stop_time*stop_time)) - net_g;

	//double stopping_acc = 2*(alt_net - (gspdv_hrz.y*stop_time))/(stop_time*stop_time);
	//if(abs(alt_net/gspdv_hrz.y) < (0.51*stop_time)) acc_req_v = (gspdv_hrz.y/stop_time) - net_g;
	//if(abs(stopping_acc) > 1) acc_req_v = (-gspdv_hrz.y/stop_time) - net_g;

	//sprintf(oapiDebugString(), "req dv %0.4f ",vvel_req-gspdv_hrz.y);
	
	//VECTOR3 tgt_vector_hrz_nm = crossp(pos_rel_glob_nm, ves_tgt_nml_glob_nm);
	VECTOR3 tgt_vector_hrz_nm = _V(tgt_vector_hrz.x,0,tgt_vector_hrz.z);
	normalise(tgt_vector_hrz_nm);

	VECTOR3 gspdv_hrz_nm = gspdv_hrz;
	normalise(gspdv_hrz_nm);

	VECTOR3 th_vector_surface_nm = _V(gspdv_hrz_nm.x,0,gspdv_hrz_nm.z);
	normalise(th_vector_surface_nm);
	VECTOR3 vel_tgt_nml = crossp(th_vector_surface_nm,tgt_vector_hrz_nm);
	double vel_tgt_angle = acos(dotp(th_vector_surface_nm,tgt_vector_hrz_nm));
	if(vel_tgt_nml.y > 0) vel_tgt_angle = -vel_tgt_angle;


	if(vel_tgt_angle > (6*RAD))vel_tgt_angle = (6*RAD);
	else if (vel_tgt_angle < (-6*RAD))vel_tgt_angle = (-6*RAD);
	MATRIX3 rm;
	rm = rotm(_V(0,1,0),10*vel_tgt_angle);

	//sprintf(oapiDebugString(), "vel_tgt_angle %0.1f",vel_tgt_angle*10*DEG);

	double th_mag = acc_req_h/(cos(2*vel_tgt_angle));
	//VECTOR3 th_vector_surface = _V((acc_req_v*tgt_vector_hrz_nm.x),0,(acc_req_v*tgt_vector_hrz_nm.z));
	VECTOR3 th_vector_surface = _V((-th_mag*gspdv_hrz_nm.x),0,(-th_mag*gspdv_hrz_nm.z));

	th_vector_surface = mul(rm,th_vector_surface);

	double hvel_req = 0;
	double tgt_dist = length(_V(tgt_vector_hrz.x,0,tgt_vector_hrz.z));

	VECTOR3 th_vector_vert = _V(0, acc_req_v, 0);
	VECTOR3 th_vector = th_vector_surface + th_vector_vert;
	double acc_req_tot = length(th_vector);
	double th_level = acc_req_tot/max_acc;

	/////////////////////////////////// final approach
	if((altmr - tgt_elev) < 15000)
	{
		tgt_range = tgt_dist;

		double tgt_off_y = 500;
		double vel_off_y = -50;
		double angle_factor = 2;
		if((altmr - tgt_elev) < 6000)angle_factor = 3;

		if((altmr - tgt_elev) < 1000)
		{
		tgt_off_y = 20;
		vel_off_y = -2;
		angle_factor = 2;
		}
		if((altmr - tgt_elev) < 30)
		{
		tgt_off_y = 2;
		vel_off_y = -0.1;
		angle_factor = 0.5;
		}


		alt_net = -altmr + tgt_elev + tgt_off_y;
		stop_time = 2*(alt_net/(gspdv_hrz.y+vel_off_y));
		acc_req_v = -((gspdv_hrz.y-vel_off_y)/stop_time) - net_g;

		VECTOR3 me_tgt = tgt_vector_hrz;
		me_tgt.y = tgt_vector_hrz.y + tgt_off_y;
		VECTOR3 me_tgt_nm = me_tgt;
		normalise(me_tgt_nm);
		VECTOR3 vel_tgt_nml = crossp(me_tgt_nm, gspdv_hrz_nm);
		normalise(vel_tgt_nml);
		double me_tgt_angle = acos(dotp(me_tgt_nm,gspdv_hrz_nm));
		th_vector = _V(0,1,0);
		rm = rotm(vel_tgt_nml,(angle_factor*me_tgt_angle));
		th_vector = mul(rm, th_vector);

		if((altmr - tgt_elev) < 4)th_vector = _V(0,1,0);
		//rm = rotm(_V(0,1,0),(0.1*vel_tgt_angle));
		//th_vector = mul(rm, th_vector);
		normalise(th_vector);

		VECTOR3 z_axis_hrz;
		HorizonRot(_V(0,0,1), z_axis_hrz);
		normalise(z_axis_hrz);

		//double th_factor = (dotp(th_vector, _V(0,1,0)));
		double th_factor = (dotp(z_axis_hrz, _V(0,1,0)));
		acc_req_v = acc_req_v/th_factor;


		th_level = acc_req_v/max_acc;



		
	//	sprintf(oapiDebugString(), "acc_req_h %0.4f stop_time %0.1f", acc_req_h, stop_time);

	
	
	}


	

	//sprintf(oapiDebugString(), "th_vector_surface %0.4f %0.4f %0.4f vel_tgt_angle %0.4f ",th_vector_surface.x, th_vector_surface.y,th_vector_surface.z, vel_tgt_angle*DEG);



	if(th_level > 1)th_level = 1;
	else if(th_level < 0)th_level = 0;

	if(th_level > 0.6)pdi_sw = 1;

	if(pdi_sw == 1)
	{
	SetThrusterGroupLevel(THGROUP_MAIN, th_level);
	}

	HorizonInvRot(th_vector, flight_vector);
	normalise(flight_vector);
	GoToFlightVector();


	

	}
}
